@@ -1,28 +1,30 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
-from __future__ import with_statement
 from trytond.model import ModelView, ModelSQL, fields
-from trytond.pyson import Not, Equal, Eval
+from trytond.pyson import Eval
 from trytond.transaction import Transaction
+from trytond.pool import Pool
 
 
 class InvoiceLine(ModelSQL, ModelView):
     _name = 'account.invoice.line'
 
     analytic_accounts = fields.Many2One('analytic_account.account.selection',
-            'Analytic Accounts',
-            states={
-                'invisible': Not(Equal(Eval('type'), 'line')),
-            })
+        'Analytic Accounts',
+        states={
+            'invisible': Eval('type') != 'line',
+            },
+        depends=['type'])
 
-    def _view_look_dom_arch(self, tree, type):
-        analytic_account_obj = self.pool.get('analytic_account.account')
+    def _view_look_dom_arch(self, tree, type, field_children=None):
+        analytic_account_obj = Pool().get('analytic_account.account')
         analytic_account_obj.convert_view(tree)
-        arch, fields = super(InvoiceLine, self)._view_look_dom_arch(tree, type)
+        arch, fields = super(InvoiceLine, self)._view_look_dom_arch(tree, type,
+            field_children=field_children)
         return arch, fields
 
     def fields_get(self, fields_names=None):
-        analytic_account_obj = self.pool.get('analytic_account.account')
+        analytic_account_obj = Pool().get('analytic_account.account')
 
         res = super(InvoiceLine, self).fields_get(fields_names)
 
@@ -33,8 +35,13 @@ class InvoiceLine(ModelSQL, ModelView):
             analytic_accounts_field, fields_names))
         return res
 
+    def default_get(self, fields, with_rec_name=True):
+        fields = [x for x in fields if not x.startswith('analytic_account_')]
+        return super(InvoiceLine, self).default_get(fields,
+            with_rec_name=with_rec_name)
+
     def read(self, ids, fields_names=None):
-        selection_obj = self.pool.get('analytic_account.account.selection')
+        selection_obj = Pool().get('analytic_account.account.selection')
 
         int_id = False
         if isinstance(ids, (int, long)):
@@ -84,7 +91,7 @@ class InvoiceLine(ModelSQL, ModelView):
         return res
 
     def create(self, vals):
-        selection_obj = self.pool.get('analytic_account.account.selection')
+        selection_obj = Pool().get('analytic_account.account.selection')
         vals = vals.copy()
         selection_vals = {}
         for field in vals.keys():
@@ -101,7 +108,7 @@ class InvoiceLine(ModelSQL, ModelView):
         return super(InvoiceLine, self).create(vals)
 
     def write(self, ids, vals):
-        selection_obj = self.pool.get('analytic_account.account.selection')
+        selection_obj = Pool().get('analytic_account.account.selection')
         vals = vals.copy()
         if isinstance(ids, (int, long)):
             ids = [ids]
@@ -142,7 +149,7 @@ class InvoiceLine(ModelSQL, ModelView):
         return super(InvoiceLine, self).write(ids, vals)
 
     def delete(self, ids):
-        selection_obj = self.pool.get('analytic_account.account.selection')
+        selection_obj = Pool().get('analytic_account.account.selection')
 
         if isinstance(ids, (int, long)):
             ids = [ids]
@@ -157,7 +164,7 @@ class InvoiceLine(ModelSQL, ModelView):
         return res
 
     def copy(self, ids, default=None):
-        selection_obj = self.pool.get('analytic_account.account.selection')
+        selection_obj = Pool().get('analytic_account.account.selection')
 
         new_ids = super(InvoiceLine, self).copy(ids, default=default)
 
@@ -177,22 +184,33 @@ class InvoiceLine(ModelSQL, ModelView):
             return new_ids[0]
         return new_ids
 
+    def _credit(self, line):
+        selection_obj = Pool().get('analytic_account.account.selection')
+
+        result = super(InvoiceLine, self)._credit(line)
+
+        if line.analytic_accounts:
+            selection_id = selection_obj.copy(line.analytic_accounts.id)
+            result['analytic_accounts'] = selection_id
+        return result
+
     def get_move_line(self, line):
-        res = super(InvoiceLine, self).get_move_line(line)
+        values = super(InvoiceLine, self).get_move_line(line)
         if line.analytic_accounts and line.analytic_accounts.accounts:
-            res['analytic_lines'] = []
-            for account in line.analytic_accounts.accounts:
-                vals = {}
-                vals['name'] = line.description
-                vals['debit'] = res['debit']
-                vals['credit'] = res['credit']
-                vals['account'] = account.id
-                vals['journal'] = line.invoice.journal.id
-                vals['date'] = line.invoice.invoice_date
-                vals['reference'] = line.invoice.reference
-                vals['party'] = line.invoice.party.id
-                res['analytic_lines'].append(('create', vals))
-        return res
+            for value in values:
+                value['analytic_lines'] = []
+                for account in line.analytic_accounts.accounts:
+                    vals = {}
+                    vals['name'] = line.description
+                    vals['debit'] = value['debit']
+                    vals['credit'] = value['credit']
+                    vals['account'] = account.id
+                    vals['journal'] = line.invoice.journal.id
+                    vals['date'] = line.invoice.invoice_date
+                    vals['reference'] = line.invoice.reference
+                    vals['party'] = line.invoice.party.id
+                    value['analytic_lines'].append(('create', vals))
+        return values
 
 InvoiceLine()
 
@@ -201,21 +219,21 @@ class Account(ModelSQL, ModelView):
     _name = 'analytic_account.account'
 
     def delete(self, ids):
-        account_invoice_line_obj = self.pool.get('account.invoice.line')
+        account_invoice_line_obj = Pool().get('account.invoice.line')
         res = super(Account, self).delete(ids)
         # Restart the cache on the fields_view_get method of account.invoice.line
         account_invoice_line_obj.fields_view_get.reset()
         return res
 
     def create(self, vals):
-        account_invoice_line_obj = self.pool.get('account.invoice.line')
+        account_invoice_line_obj = Pool().get('account.invoice.line')
         res = super(Account, self).create(vals)
         # Restart the cache on the fields_view_get method of account.invoice.line
         account_invoice_line_obj.fields_view_get.reset()
         return res
 
     def write(self, ids, vals):
-        account_invoice_line_obj = self.pool.get('account.invoice.line')
+        account_invoice_line_obj = Pool().get('account.invoice.line')
         res = super(Account, self).write(ids, vals)
         # Restart the cache on the fields_view_get method of account.invoice.line
         account_invoice_line_obj.fields_view_get.reset()
